/*********************************/
/*    ナダ電子 AS-289R テスト02  */
/*********************************/
//    2014-04-21    下間憲行
//    P_ZERがチャートのゼロ点位置　要調整

/*****  ライブラリ      ****/
#include  "wiring_private.h"       // sbi,cbi命令用
// #include  <MsTimer2.h>          // 0.5msに対応していないので使わない
// #include  <avr/io.h>
// #include  <avr/pgmspace.h>      // PROGMEME用
// #include  <avr/interrupt.h>     // 割り込み用

/*****  I/Oピン指定    *****/
#define  SW_START  6        // start/stop SW
#define  SW_CH1    12       // CH1 on/off
#define  SW_CH2    13       // CH2 on/off
                            // いずれもHでオン
#define  SW_TEST   A4       // テスト印字
                            // Lでテスト
/*****  マクロ        *****/
#define  DIMSIZ(a)  (sizeof(a) / sizeof(*a))  // 配列の大きさ

/*****  ピン制御マクロ    *****/
//      タイミングチェックパルス出力
#define  TP2_H    (sbi(PORTD,PORTD2))    // 1ms割り込み
#define  TP2_L    (cbi(PORTD,PORTD2))
#define  TP3_H    (sbi(PORTD,PORTD3))    // A/D割り込み
#define  TP3_L    (cbi(PORTD,PORTD3))
#define  TP4_H    (sbi(PORTD,PORTD4))    // loop実行時間
#define  TP4_L    (cbi(PORTD,PORTD4))
#define  TP4_TGL  (sbi(PIND, PIND4))

/******************************/
/*      イメージプリント処理  */
/******************************/

/*****  プリントデータバッファ    *****/
#define  P_BYTE    48              // 1行バイト数 384ドットのバイト数
byte  p_bff[P_BYTE];               // 1行分のバッファ
                                   // これをプリンタに送出
/*****  イメージプリントコマンド  *****/
PROGMEM prog_uchar pcmd_bitimg[]={
    0x1c, 0x2a, 0x63,      // イメージ登録+プリント
    0x00, 0x01,            // 1line
};

/*****  1ラインバッファ出力      *****/
void prnpbff(void)
{
byte i;
    for(i = 0; i < sizeof(pcmd_bitimg); i++){  // コマンド出力
        Serial.write(pgm_read_byte(&pcmd_bitimg[i]));
    }
    for(i = 0; i < P_BYTE; i++){          // バッファ出力
        Serial.write(p_bff[i]);          // 1行
    }
}

/*****  ビットイメージデータ  *****/
PROGMEM prog_uchar bit_on[]={
    0b10000000,       // 0
    0b01000000,       // 1
    0b00100000,       // 2
    0b00010000,       // 3
    0b00001000,       // 4
    0b00000100,       // 5
    0b00000010,       // 6
    0b00000001,       // 7
};

/*****  ビットイメージデータ(左側）  *****/
PROGMEM prog_uchar bit_on1[]={
    0b11111111,       // 0
    0b01111111,       // 1
    0b00111111,       // 2
    0b00011111,       // 3
    0b00001111,       // 4
    0b00000111,       // 5
    0b00000011,       // 6
    0b00000001,       // 7
};

/*****  ビットイメージデータ(右側）  *****/
PROGMEM prog_uchar bit_on2[]={
    0b10000000,       // 0
    0b11000000,       // 1
    0b11100000,       // 2
    0b11110000,       // 3
    0b11111000,       // 4
    0b11111100,       // 5
    0b11111110,       // 6
    0b11111111,       // 7
};

/*****  ドットセット      *****/
//    左上 0,0　　383,0 右上
//    左下 0,1　　383,1 右下
void pset(int x)
{
      p_bff[x / 8] |=
          pgm_read_byte(&bit_on[x % 8]);    // 1ビットオン
}

/*****  プリントバッファクリア      *****/
void clrpbff(void)
{
    memset(p_bff, 0, sizeof(p_bff));
}

/**********************************/
/*    プリント、紙送り処理        */
/**********************************/

/*****  紙送りデータ    *****/
#define  CM_MS    125      // 5秒/1cmの基準パルス
                           // 5秒/80パルス = 0.0625秒
                           // 0.5ms割り込みで125回
volatile byte tm_feed;     // 紙送り基準ﾀｲﾏｰ
                           // 1msでアップカウント
volatile word feed_cnt;    // feedplsと比較して1ピッチ紙送り
volatile byte f_feed;      // 紙送りフラグ
                           // タイマー割り込み内でオン
byte feed_8;               // 紙送り1mmチェック
                           // 1mmで8パルス
word feed_mm;              // 紙送りmm単位の長さ
                           // 00～999mmを繰り返す
byte  f_swstart;           // SW_STARTの状態
byte  f_swston;            // START オンのエッジ
byte  dsw_rot;             // 4bit DSWのデータ 0～9
byte  f_dswrotx;           // DSWデータが変化したときにオン
volatile byte  tm_swstck;  // SW_STARTﾁｪｯｸ用ﾀｲﾏｰ
volatile byte  tm_dswck;   // DSW ﾁｪｯｸ用ﾀｲﾏｰ

/*****  ROT DSWの入力    *****/
//    0～9を入力
byte rddsw(void)
{
byte d;
    d = (~PINB) & 0x0F;    // PB0～PB3(MSB)
    if(d > 9)    d = 0;
    return d;
}

/*****  紙送り速度      *****/
//    5秒/cmの紙送りパルス(62.5ms)を単位として指定
PROGMEM prog_uint16_t feedpls[]={
      1,        // 0  5秒/cm
      2,        // 1  10秒/cm
      6,        // 2  30秒/cm
      12,       // 3   1分/cm
      36,       // 4   3分/cm
      120,      // 5  10分/cm
      360,      // 6  30分/cm
      720,      // 7   1時/cm
      1440,     // 8   2時/cm
      4320,     // 9   6時/cm
};

/*****  紙送り速度表示データ      *****/
PROGMEM prog_char spd0[] = "5sec/cm";
PROGMEM prog_char spd1[] = "10sec/cm";
PROGMEM prog_char spd2[] = "30sec/cm";
PROGMEM prog_char spd3[] = "1min/cm";
PROGMEM prog_char spd4[] = "3min/cm";
PROGMEM prog_char spd5[] = "10min/cm";
PROGMEM prog_char spd6[] = "30min/cm";
PROGMEM prog_char spd7[] = "1hour/cm";
PROGMEM prog_char spd8[] = "2hour/cm";
PROGMEM prog_char spd9[] = "6hour/cm";

PROGMEM prog_char *spdmsg[] = {
      spd0,  spd1,  spd2,  spd3,  spd4,
      spd5,  spd6,  spd7,  spd8,  spd9,
};

//    用紙スケール　　0～383の384ドット
//    |                   .                   | 0.5mm
//    |   .   .   .   .   .   .   .   .   .   | 1mm
//    | . . . . . . . . . . . . . . . . . . . | 10mm

#define  P_SCL    382        // スケール上限
#define  P_ZER      7        // スケール ゼロ位置（左端）
#define  P_PIT     36        // 1mmごと 1/10スケール
#define  P_CEN    (P_ZER + (5 * P_PIT))     // 中央
#define  P_FUL    (P_ZER + (10 * P_PIT))    // 右端

/*****   スケールを描画        *****/
void pscale(void)
{
int i;
    pset(P_ZER);                    // 左端:ゼロ点
    pset(P_FUL);                    // 右端
    if((feed_8 == 0) && ((feed_mm % 10) == 0)){   // 10mmピッチ
        for(i = 0; i < P_BYTE; i++){              // 1cmのライン
            p_bff[i] = 0b11001100;
        }
    }  
    else if(feed_8 == 0){          // 1mmピッチ
        for(i = P_ZER + P_PIT; i < P_FUL; i+=P_PIT){
            pset(i);
        }
    }
    else if((feed_8 % 4) == 0){     // 0.5mmピッチ
        pset(P_CEN);                // 中央の点
    }     
}

/******************************/
/*      Ａ／Ｄ処理            */
/******************************/

/****  A/D データ        *****/
//    AD0  ch1データ入力
//    AD1  ch2データ入力
volatile byte  ad_ch;           // A/D変換チャンネル
                                // 0,1 : min,max処理
//  AD0,AD1 データ (ch1,ch2,データ)
volatile int   ad_data[2];     // A/D変換データ 0～1023
volatile int   ad_max[2];      // A/D max
volatile int   ad_min[2];      // A/D min

//  A/D min,max更新フラグ
volatile byte f_adnext;         // A/Dmin,maxを次データに
                                // メイン側でオンして割り込みでクリア

/*****  A/D 変換チャンネル指定    *****/
PROGMEM prog_uchar ad_mux[]={
    0b11000000,       // ch0 入力1
    0b11000001,       // ch1 入力2
//    ||| ++++----  MUX ch
//    ||+---------  ADLAR
//    ++----------  REFS   内部基準電圧1.1V
};

/*****  A/D割り込み処理    *****/
//    タイマー割り込みでA/D変換開始
//    1msで2chの変換を終わる
//    0,1は入力データ
//    min,max判定のためこの処理の中で
ISR(ADC_vect)
{
int d;
    TP3_H;                      // (!!!)test
    d = (word)ADCL;             // 
    d |= (ADCH & 0x03) << 8;    // 符号なしで 0～3FF
//  測定値
    ad_data[ad_ch] = d;             // データを保存
    if(d > ad_max[ad_ch])    ad_max[ad_ch] = d;    // max
    if(d < ad_min[ad_ch])    ad_min[ad_ch] = d;    // min
//  2ch変換完了
    ad_ch++;                          // 次ch
    if(ad_ch >= sizeof(ad_mux)){      // 2ch終了
        ad_ch = 0;                    // ch0に戻す
//  AD0,1 : フラグがあれば次データに更新
        if(f_adnext){           // 次変換データの処理
          ad_max[0] = ad_min[0] = ad_data[0];    // ch1入力
          ad_max[1] = ad_min[1] = ad_data[1];    // ch2入力
          f_adnext = 0;
        }
    }
    TP3_L;
}

/******************************/
/*      タイマー割り込み      */
/******************************/

/*****  タイマー値    *****/
volatile byte  tm_1ms;       // 1msﾀﾞｳﾝｶｳﾝﾄﾀｲﾏｰ max 255ms

/*****  タイマー2 ０．５ｍＳ割り込み    *****/
//      A/Dに対して変換指令
ISR(TIMER2_COMPA_vect)
{
static byte tc = 0;                 // 1msカウント 
    TP2_H;                          // (!!!)test
//  0.5ms 紙送りタイマー
    if(f_swstart == HIGH){          // start/stop SW
      tm_feed++;                    // 1msアップカウント
        if(tm_feed >= CM_MS){       // 63.5msが基準 
          tm_feed = 0;
          feed_cnt++;             // 送りカウンタ
          if(feed_cnt >= pgm_read_word(&feedpls[dsw_rot])){  // DSW設定値
            feed_cnt = 0;
            f_feed = 1;         // 紙送りフラグをオン
          }
        }
    }
//  0.5msを2回カウントして1msに
    tc++;                    // 0,1
    if(tc >= 2){             // 1ms
      tc = 0;  
//  1ms計時処理（ダウンカウント）
      if(tm_1ms)      tm_1ms--;
      if(tm_swstck)   tm_swstck--;
      if(tm_dswck)    tm_dswck--;
    }
//  A/D変換　データの読み出しは変換完了割り込みで
    ADMUX = pgm_read_byte(&ad_mux[ad_ch]);  // ch指定
    ADCSRA |= (1 << ADSC);         // A/D変換開始
    TP2_L;                         // (!!!)test
}

/****************************************/
/*    スイッチ入力チェック              */
/****************************************/

/*****  START/STOPスイッチチェック    *****/
//    onのエッジを見つける
void ckswstart(void)
{
static byte ck = 0;
byte d;
    d = digitalRead(SW_START);
    switch(ck){
      case 0:
        if(d != f_swstart){    // 変化あり
          tm_swstck = 10;      // 10ms
          ck++;
        }
        break;
      case 1:
        if(d == f_swstart){    // 元に戻った
          ck = 0;
        }
        else{                         // 変化継続
          if(tm_swstck == 0){         // 10ms経過
            f_swstart = d;            // 新状態
            if(d == HIGH){
              f_swston = 1;           // onのエッジ
            }
            ck = 0;
          }
        }
        break;
    }
}

/*****  ROT-DSW入力      *****/
//    dsw_rotにデータを残す
void ckrotdsw(void)
{
static byte ck = 0;
static byte d;
    switch(ck){
      case 0:
        if(dsw_rot != rddsw()){    // 変化あり
          d = rddsw();             // 変化した状態
          tm_dswck = 100;          // 100ms
          ck++;
        }
        break;
      case 1:
        if(d != rddsw()){          // また変わった
          ck = 0;                  // 最初から
        }
        else{                       // 同じ状態継続
          if(tm_dswck == 0){        // 100ms経過
            dsw_rot = d;            // 新状態
            f_dswrotx = 1;          // 変化ありを知らせる
            ck = 0;
          }
        }
        break;
    }
}

/******************************/
/*    A/D 値スケーリング      */
/******************************/

/*****  A/D値からX位置を計算    *****/
//    in  a:A/D 0～1023
//    ret X位置 0～380
int xpos(int a)
{
long x;
    x  = (long)a * (long)P_SCL;  // 用紙幅とスケーリング
    x /= 1024L;                  // 10bit
    if(x > P_SCL)    x = P_SCL;
    if(x < 0)        x = 0;
    return (int)x;
}

/*****  グラフ描画      *****/
//      x1,x2 座標
void pgrph(int x1, int x2)
{
#if 0      // psetでの描画
int  a;
    if(x1 > x2){
        a  = x1;
        x1 = x2;
        x2 = a;
    }
    a = x1;
    for(a = x1; a <= x2; a++){
      pset(a);
    }
#else       // 左・中・右に分けて描画
int i, a1,b1, a2,b2;
    if(x1 > x2){        // x2を大きく
        a1 = x1;
        x1 = x2;
        x2 = a1;
    }
    if(x1 == x2)    x2+=1;    // 同じ点なら左へ拡張
// アドレスとビット位置
    a1 = x1 / 8;
    b1 = x1 % 8;
    a2 = x2 / 8;
    b2 = x2 % 8;
//  同じアドレスか
    if(a1 == a2){        // 同じアドレス
        p_bff[a1] |= 
          pgm_read_byte(&bit_on1[b1]) &
          pgm_read_byte(&bit_on2[b2]);      // 左右ビットをANDしてOR
    }
    else{               // 違うアドレス
        p_bff[a1] |= pgm_read_byte(&bit_on1[b1]); // 左
        p_bff[a2] |= pgm_read_byte(&bit_on2[b2]); // 右ビット
        if((a2 - a1) > 1){                        // 中間がある
          for(i = (a1 + 1); i < a2; i++){         // 間はFFで埋める
            p_bff[i] |= 0xff;
          }
        }
    }
#endif
}

/*****  紙送り速度を出力    *****/
void prnspeed(void)
{
char s[16];
    strcpy_P(s,(PGM_P)pgm_read_word(&spdmsg[dsw_rot]));  // 設定値
    Serial.print(s);              // 送り速度
    Serial.write('\r');           // 印字と改行
}

/**********************************/
/*    キャラジェネ                */
/**********************************/

/***** 数字 0～9    *****/
//    横16ドット 高8ドット 横向きで
PROGMEM prog_uint16_t cg09[]={
    // 0
    0b0011111111111100,
    0b0111111111111110,
    0b1100000000000011,
    0b1100000000000011,
    0b1100000000000011,
    0b1100000000000011,
    0b0111111111111110,
    0b0011111111111100,
    // 1
    0b0000000000000000,
    0b0000000000010000,
    0b1100000000011100,
    0b1111111111111111,
    0b1111111111111111,
    0b1100000000000000,
    0b0000000000000000,
    0b0000000000000000,
    // 2
    0b1111000000011100,
    0b1111100000011110,
    0b1100110000000011,
    0b1100011000000011,
    0b1100001100000011,
    0b1100000110000011,
    0b1100000011111110,
    0b1100000001111100,
    // 3
    0b0011000000001100,
    0b0111000000001110,
    0b1100000110000011,
    0b1100000110000011,
    0b1100000110000011,
    0b1100001111000011,
    0b0111111001111110,
    0b0011111000111100,
    // 4
    0b0000011110000000,
    0b0000011111100000,
    0b0000011001111000,
    0b0000011000011100,
    0b0000011000000110,
    0b1111111111111111,
    0b1111111111111111,
    0b0000011000000000,
    // 5
    0b0011000001111111,
    0b0111000011111111,
    0b1100000011000011,
    0b1100000011000011,
    0b1100000011000011,
    0b1100000011000011,
    0b0111111111000011,
    0b0011111110000011,
    // 6
    0b0011111111111100,
    0b0111111111111110,
    0b1100000110000011,
    0b1100000110000011,
    0b1100000110000011,
    0b1100000110000011,
    0b0111111100000110,
    0b0011111000000100,
    // 7
    0b0000000000000011,
    0b0000000000000011,
    0b0000000000000011,
    0b1111110000000011,
    0b1111111110000011,
    0b0000001111100011,
    0b0000000001111111,
    0b0000000000011111,
    // 8
    0b0011111001111100,
    0b0111111111111110,
    0b1100000110000011,
    0b1100000110000011,
    0b1100000110000011,
    0b1100000110000011,
    0b0111111111111110,
    0b0011111001111100,
    // 9
    0b0110000001111100,
    0b0110000011111110,
    0b1100000110000011,
    0b1100000110000011,
    0b1100000110000011,
    0b1100000110000011,
    0b0111111111111110,
    0b0011111111111100,
};

/********************************/
/*    １ｃｍ単位プリント        */
/********************************/

/*****  1cmピッチプリント用データ  *****/
word p1cm_cg[2+8+2];  // 1cmピッチプリント用バッファ
                      // 2:空白 8:データ 2:空白
                      // 10桁と1桁は別タイミングでプリント
byte p1cm_ptr;        // lineバッファへのコピー位置
byte p1cm_cnt;        // コピーデータ数
                      // 0以外なら紙送りのタイミングで
                      // グラフの右端にプリント

/*****  紙送り1cmピッチプリント開始  *****/
//  1cmごとにプリント m:0～9
void prn1cmset(word m)
{
    memcpy_P(p1cm_cg+2, &cg09[m * 8], 16);  // 1桁
    p1cm_cnt = 12;        // 空白4ドット,データ8ドット
    p1cm_ptr = 0;         // 先頭ライン
}

/*****  紙送り1cmピッチプリント実行  *****/
//      lineバッファの右端(+45,+46,+47)にコピー
void prn1cmexc(void)
{
    p_bff[P_BYTE-1] = lowByte(p1cm_cg[p1cm_ptr]);    // LSB CG 左端
    p_bff[P_BYTE-2] = highByte(p1cm_cg[p1cm_ptr]);   // MSB CG 1/2右
    p_bff[P_BYTE-3] &= 0b11110000;                   // 364～367
    p1cm_ptr++;                      // 次行へ
    p1cm_cnt--;                      // カウンタ -1
}

/********************************/
/*    ch番号プリント            */
/********************************/

/*****  ch番号プリント用データ  *****/
word pch_cg[8];      // ch番号キャラジェネ
byte pch_ptr;        // lineバッファへのコピー位置
byte pch_cnt;        // コピーデータ数
                     // 0以外なら紙送りのタイミングでプリント
byte pch_pos;        // 印字位置(0～47)

/*****  ch番号プリント開始  *****/
//    x1:グラフ描画左端位置(min) x2:右端(max)
void prnchset(int x1,int x2, byte ch)
{
    memcpy_P(pch_cg, &cg09[ch * 8], 16);  // 1桁
    pch_cnt = 8;        // データ8ドット
    pch_ptr = 0;        // 先頭ライン
    if(x2 < 336){       // max位置 42コラム
        pch_pos = ((x2 | 0x0007) + 1) / 8;
    }
    else if(x1 >= 16){  // min位置
        pch_pos = ((x1 & 0xfff8) - 16) / 8;
    }
    else{
        pch_pos = 0;
    }
}

/*****  ch番号プリント実行  *****/
//      pch_posの位置にコピー
void prnchexc(void)
{
    p_bff[pch_pos+1] =  lowByte(pch_cg[pch_ptr]);   // LSB CG 左端
    p_bff[pch_pos]   = highByte(pch_cg[pch_ptr]);   // MSB CG 1/2右
    pch_ptr++;                      // 次行へ
    pch_cnt--;                      // カウンタ -1
}

/************************************/
/*    グラフ出力                    */
/************************************/

/*****  グラフデータプリント    *****/
//      紙送り指令でプリント
void prndata(void)
{
int i, x1, x2;
int d_now[2];        // ad_data
int d_max[2];        // max
int d_min[2];        // min
                     // A/D 測定データ ch1,ch2
    clrpbff();             // プリントバッファをクリア
    pscale();              // スケールを描画
//  紙送りcmをプリント
    if((feed_8 == 4) && ((feed_mm % 10) == 8)){  // 8mmで10桁
        prn1cmset((((feed_mm / 10) +1) % 100)  / 10);
    }
    if((feed_8 == 1) && ((feed_mm % 10) == 0)){  // 0mmで1桁
        prn1cmset((feed_mm / 10) % 10);          // CGデータをセット
    }
    if(p1cm_cnt)           prn1cmexc();    // cmプリント実行
//  ch番号をプリント
    if(pch_cnt)            prnchexc();     // ch番号プリント
//  ch1,ch2データ読み出しとプリント        
    cli();                         // 割り込み禁止にして
    for(i = 0; i < 2; i++){        // 2ch分のA/Dデータを取得
        d_max[i] = ad_max[i];     // maxとmin
        d_min[i] = ad_min[i];
        d_now[i] = ad_max[i] = ad_min[i] = ad_data[i];  // 新データに
    }
    sei();                         // 割り込み再開
    if(digitalRead(SW_CH1) == HIGH){         // ch1
        x1 = xpos(d_min[0]);
        x2 = xpos(d_max[0]);
        pgrph(x1, x2);                       // グラフ
        if((feed_8 == 0) && ((feed_mm % 100) == 5)){  // 5mm位置
            prnchset(x1, x2, 1);             // ch番号
        }
    }
    if(digitalRead(SW_CH2) == HIGH){         // ch2
        x1 = xpos(d_min[1]);
        x2 = xpos(d_max[1]);
        pgrph(x1, x2);                       // グラフ
        if((feed_8 == 0) && ((feed_mm % 100) == 15)){  // 15mm位置
            prnchset(x1, x2, 2);             // ch番号
        }
    }
//  バッファを出力
    TP4_L;                 // (!!!)test
    TP4_H;
    if(digitalRead(SW_TEST) == LOW){  // テスト印字
          Serial.print(d_now[0]);       // ch1データ
          Serial.print(" ");
          Serial.println(d_now[1]);     // ch2データ
    }
    else{                      // test sw オフなら  
          prnpbff();           // バッファをプリント
    }
//  紙送りピッチ
    feed_8++;                    // 紙送りピッチ
    if(feed_8 >= 8){             // 1mm
        feed_8 = 0;
        feed_mm++;
        if(feed_mm >= 1000){
            feed_mm = 0;         // 0～999mm
        }
    }
}

/******************************/
/*    セットアップとループ    */
/******************************/

/*****  セットアップ    ****/
void setup()
{
    Serial.begin(9600);
    pinMode(2,OUTPUT);        // PD2
    pinMode(3,OUTPUT);        // PD3
    pinMode(4,OUTPUT);        // PD4
    pinMode(SW_START,INPUT_PULLUP);  // START/STOP
    pinMode(SW_CH1,INPUT_PULLUP);    // CH1 on/off
    pinMode(SW_CH2,INPUT_PULLUP);    // CH2 on/off
    pinMode(SW_TEST,INPUT_PULLUP);   // test print on/off
    pinMode(8,INPUT_PULLUP);  // ROT DSW 1 PIB0
    pinMode(9,INPUT_PULLUP);  // ROT DSW 2 PINB1
    pinMode(10,INPUT_PULLUP); // ROT DSW 4 PINB2
    pinMode(11,INPUT_PULLUP); // ROT DSW 8 PINB3
//  A/D割り込み
    ADCSRA |= (1 << ADEN);      // A/D許可
    ADCSRA |= (1 << ADIE);      // A/D割り込み有効
//  ﾀｲﾏｰ計時
//    MsTimer2::set(1, tm2int);   // 1msｻｲｸﾙ割り込み処理
//    MsTimer2::start();
    TCCR2A = 0b00000010;
    //               ++----  WGM21,20 CTCモード
    TCCR2B = 0b00000100;
    //             |+++----  プリスケーラ1/64
    //             +-------  WGM22
    OCR2A  = 125-1;          // 16MHz / 64 / 125 = 2kHz
    TIMSK2 = 0b00000010;
    //               +-----  OCIE2A
    sei();                     // 割り込み開始
//  最初の測定
    tm_1ms = 100;              // 0.1sec待ち
    while(tm_1ms);
    f_adnext = 1;              // A/D min,max更新指令
    while(f_adnext){           // 2ms経過して更新完了までwait
        TP4_TGL;               // (!!!)test
    }
    f_feed = 0;                // 最初の紙送りを無視
//  紙送り速度をプリント
    dsw_rot = rddsw();
    f_swstart = digitalRead(SW_START);    // 紙送り状態
    if(f_swstart == HIGH){      // start状態
        prnspeed();             // 紙送り速度をプリント
    }
}

/*****  ルー プ      *****/
void loop()
{
//  スイッチチェック
    ckswstart();               // STARTスイッチ状態
    if(f_swston){              // onした
        f_swston = 0;
        prnspeed();            // 紙送り速度をプリント
        f_adnext = 1;          // A/D min,max更新指令
        while(f_adnext);       // 2ms経過して更新完了までwait
        feed_8   = 0;          // 0mmから
        feed_mm  = 0;
        f_feed   = 0;          // 紙送りをキャンセル
        p1cm_cnt = 0;          // 1cmスケール印字をキャンセル
        pch_cnt  = 0;          // ch番号印字をキャンセル
    }
    ckrotdsw();                // 速度設定DSWの変化をチェック
    if(f_dswrotx){             // 変化があれば
        f_dswrotx = 0;
        if(f_swstart == HIGH){   // 印刷中なら
          prnspeed();            // 紙送り速度をプリント
        }
    }
//  紙送り,プリント
    if(f_feed){                // 紙送り指令
        TP4_H;                 // (!!!)test
        f_feed = 0;
        prndata();             // グラフデータをプリント
        TP4_L;
    }
}

/*=====  end of prnsld02    =====*/

